@@ -2,6 +2,10 @@ import struct
 from enum import Enum
 import sys
 import math
+from .encoding import encode as base_pack, decode as unpack
+
+def pack(fmt, *args):
+    return base_pack(fmt, args)
 
 class SoftDecodeFailure(RuntimeError):
     pass
@@ -62,13 +66,13 @@ class VersionPacket:
         self.patch = patch
 
     def encode(self):
-        return struct.pack('<IfIII', 0,
-                                     float('{}.{}'.format(self.major, self.minor)),
-                                    self.major, self.minor, self.patch)
+        return pack('IfIII', 0,
+                     float('{}.{}'.format(self.major, self.minor)),
+                     self.major, self.minor, self.patch)
 
     @classmethod
     def decode(cls, packet):
-        unknown_1, legacy_version, major, minor, patch = struct.unpack('<IfIII', packet)
+        unknown_1, legacy_version, major, minor, patch = unpack('IfIII', packet)
         return cls(major, minor, patch)
 
     def __str__(self):
@@ -89,11 +93,11 @@ class DifficultyPacket:
         self.game_type = game_type
 
     def encode(self):
-        return struct.pack('<II', self.difficulty, self.game_type.value)
+        return pack('II', self.difficulty, self.game_type.value)
 
     @classmethod
     def decode(cls, packet):
-        difficulty, game_type_raw = struct.unpack('<II', packet)
+        difficulty, game_type_raw = unpack('II', packet)
         return cls(difficulty, GameType(game_type_raw))
 
     def __str__(self):
@@ -123,12 +127,13 @@ class ConsoleStatusPacket:
         self.ship = ship
 
     def encode(self):
-        return struct.pack('<I', self.ship) + bytes([self.consoles[console].value for console in Console])
+        return pack('I[B]', self.ship,
+                    [(self.consoles[console].value,) for console in Console])
 
     @classmethod
     def decode(cls, packet):
-        ship, = struct.unpack('<I', packet[:4])
-        body = packet[4:]
+        ship, body = unpack('I[B]', packet)
+        body = [x[0] for x in body]
         if len(body) != len(Console):
             raise ValueError("Incorrect console count ({}, should be {})".format(len(body), len(Console)))
         consoles = {console: ConsoleStatus(body[console.value]) for console in Console}
@@ -154,13 +159,6 @@ class HeartbeatPacket:
     def __str__(self):
         return "<HeartbeatPacket>"
 
-def pack_string(s):
-    packed = s.encode('utf-16le') + b'\x00\x00'
-    return struct.pack('<I', len(packed) // 2) + packed
-
-def unpack_string(s):
-    return s[4:-2].decode('utf-16le')
-
 @packet(0xee665279)
 class IntelPacket:
     def __init__(self, object, intel):
@@ -168,12 +166,11 @@ class IntelPacket:
         self.intel = intel
 
     def encode(self):
-        return struct.pack('<Ib', self.object, 3) + pack_string(self.intel)
+        return pack('Ibu', self.object, 3, self.intel)
 
     @classmethod
     def decode(cls, packet):
-        object, unknown_1 = struct.unpack('<Ib', packet[:5])
-        intel = unpack_string(packet[5:])
+        object, _unk, intel = unpack('Ibu', packet)
         return cls(object, intel)
 
     def __str__(self):
@@ -260,15 +257,12 @@ class DmxPacket(GameMessagePacket):
         self.state = state
 
     def encode(self):
-        return (struct.pack('<I', 0x10) +
-                pack_string(self.flag) +
-                struct.pack('<I', int(self.state)))
+        return pack('IuI', 0x10, self.flag, int(self.state))
 
     @classmethod
     def decode(cls, packet):
-        data = unpack_string(packet[4:-4])
-        flag, = struct.unpack('<I', packet[-4:])
-        return cls(data, bool(flag))
+        _id, flag, state = unpack('IuI', packet)
+        return cls(flag, state)
 
     def __str__(self):
         return '<DmxPacket flag={0!r} state={1!r}>'.format(self.flag, self.state)
@@ -278,11 +272,12 @@ class PopupPacket(GameMessagePacket):
         self.message = message
 
     def encode(self):
-        return b'\x0a\x00\x00\x00' + pack_string(self.message)
+        return pack('Iu', 0x0a, self.message)
 
     @classmethod
     def decode(cls, packet):
-        return cls(unpack_string(packet[4:]))
+        _id, message = unpack('Iu', packet)
+        return cls(message)
 
     def __str__(self):
         return '<PopupPacket message={0!r}>'.format(self.message)
@@ -292,11 +287,11 @@ class AutonomousDamconPacket(GameMessagePacket):
         self.autonomy = autonomy
 
     def encode(self):
-        return struct.pack('<II', 0x0b, int(self.autonomy))
+        return pack('II', 0x0b, int(self.autonomy))
 
     @classmethod
     def decode(cls, packet):
-        id_, autonomy = struct.unpack('<II', packet)
+        _id, autonomy = unpack('II', packet)
         return cls(bool(autonomy))
 
     def __str__(self):
@@ -410,11 +405,11 @@ class ClimbDivePacket(ShipAction1Packet):
         self.direction = direction
 
     def encode(self):
-        return struct.pack('<Ii', 27, self.direction)
+        return pack('Ii', 27, self.direction)
 
     @classmethod
     def decode(cls, packet):
-        _idx, direction = struct.unpack('<Ii', packet)
+        _id, direction = unpack('Ii', packet)
         return cls(direction)
 
     def __str__(self):
@@ -425,11 +420,11 @@ class SetMainScreenPacket(ShipAction1Packet):
         self.screen = screen
 
     def encode(self):
-        return struct.pack('<II', 1, self.screen.value)
+        return pack('II', 1, self.screen.value)
 
     @classmethod
     def decode(cls, packet):
-        _idx, screen_id = struct.unpack('<II', packet)
+        _idx, screen_id = unpack('II', packet)
         return cls(MainView(screen_id))
 
     def __str__(self):
@@ -441,11 +436,11 @@ class SetConsolePacket(ShipAction1Packet):
         self.selected = selected
 
     def encode(self):
-        return struct.pack('<III', 0x0e, self.console.value, 1 if self.selected else 0)
+        return pack('III', 0x0e, self.console.value, 1 if self.selected else 0)
 
     @classmethod
     def decode(cls, packet):
-        _idx, console_id, selected = struct.unpack('<III', packet)
+        _idx, console_id, selected = unpack('III', packet)
         return cls(Console(console_id), bool(selected))
 
     def __str__(self):
@@ -456,11 +451,11 @@ class HelmSetWarpPacket(ShipAction1Packet):
         self.warp = warp
 
     def encode(self):
-        return struct.pack('<II', 0, self.warp)
+        return pack('II', 0, self.warp)
 
     @classmethod
     def decode(cls, packet):
-        _idx, warp = struct.unpack('<II', packet)
+        _idx, warp = unpack('II', packet)
         return cls(warp)
 
     def __str__(self):
@@ -471,11 +466,11 @@ class SetShipPacket(ShipAction1Packet):
         self.ship = ship
 
     def encode(self):
-        return struct.pack('<II', 0x0d, self.ship)
+        return pack('II', 0x0d, self.ship)
 
     @classmethod
     def decode(cls, packet):
-        _idx, ship = struct.unpack('<II', packet)
+        _idx, ship = unpack('II', packet)
         return cls(ship)
 
     def __str__(self):
@@ -501,11 +496,11 @@ class HelmSetSteeringPacket(ShipAction3Packet):
         self.rudder = rudder
 
     def encode(self):
-        return struct.pack('<If', 1, self.rudder)
+        return pack('If', 1, self.rudder)
 
     @classmethod
     def decode(cls, packet):
-        _idx, rudder = struct.unpack('<If', packet)
+        _idx, rudder = unpack('If', packet)
         return cls(rudder)
 
     def __str__(self):
@@ -516,11 +511,11 @@ class HelmSetImpulsePacket(ShipAction3Packet):
         self.impulse = impulse
 
     def encode(self):
-        return struct.pack('<If', 0, self.impulse)
+        return pack('If', 0, self.impulse)
 
     @classmethod
     def decode(cls, packet):
-        _idx, impulse = struct.unpack('<If', packet)
+        _idx, impulse = unpack('If', packet)
         return cls(impulse)
 
     def __str__(self):
@@ -532,11 +527,11 @@ class HelmJumpPacket(ShipAction3Packet):
         self.distance = distance
 
     def encode(self):
-        return struct.pack('<Iff', 4, self.bearing / (math.pi * 2), self.distance / 50)
+        return pack('Iff', 4, self.bearing / (math.pi * 2), self.distance / 50)
 
     @classmethod
     def decode(cls, packet):
-        _idx, bearing, distance = struct.unpack('<Iff', packet)
+        _idx, bearing, distance = unpack('Iff', packet)
         return cls(bearing * (math.pi * 2), distance * 50)
 
     def __str__(self):
